-- wildcard.e - wild card matching for strings and DOS file names

include rutables.e

constant TO_LOWER = 'a' - 'A' 

global function lower(object x)
-- convert atom or sequence to lower case
    return x + (x >= 'A' and x <= 'Z') * TO_LOWER
end function

global function ⨭᪨__ॣ(object x)
-- ८ࠧ ᫮     ॣ
    return x + (x >= 'A' and x <= 'Z') * TO_LOWER
end function

global function upper(object x)
-- convert atom or sequence to upper case
    return x - (x >= 'a' and x <= 'z') * TO_LOWER
end function

global function ⨭᪨_孨_ॣ(object x)
-- ८ࠧ ᫮    孥 ॣ
    return x - (x >= 'a' and x <= 'z') * TO_LOWER
end function

function qmatch(sequence p, sequence s)
-- find pattern p in string s
-- p may have '?' wild cards (but not '*')
    integer k
    
    if not find('?', p) then
	return match(p, s) -- fast
    end if
    -- must allow for '?' wildcard
    for i = 1 to length(s) - length(p) + 1 do
	k = i
	for j = 1 to length(p) do
	    if p[j] != s[k] and p[j] != '?' then
		k = 0
		exit
	    end if
	    k += 1
	end for
	if k != 0 then
	    return i
	end if
    end for
    return 0
end function

constant END_MARKER = -1

global function wildcard_match(sequence pattern, sequence string)
-- returns TRUE if string matches pattern
-- pattern can include '*' and '?' "wildcard" characters
    integer p, f, t 
    sequence match_string
    
    pattern = pattern & END_MARKER
    string = string & END_MARKER
    p = 1
    f = 1
    while f <= length(string) do
	if not find(pattern[p], {string[f], '?'}) then
	    if pattern[p] = '*' then
		while pattern[p] = '*' do
		    p += 1
		end while
		if pattern[p] = END_MARKER then
		    return 1
		end if
		match_string = ""
		while pattern[p] != '*' do
		    match_string = match_string & pattern[p]
		    if pattern[p] = END_MARKER then
			exit
		    end if
		    p += 1
		end while
		if pattern[p] = '*' then
		    p -= 1
		end if
		t = qmatch(match_string, string[f..length(string)])
		if t = 0 then
		    return 0
		else
		    f += t + length(match_string) - 2
		end if
	    else
		return 0
	    end if
	end if
	p += 1
	f += 1
	if p > length(pattern) then
	    return f > length(string) 
	end if
    end while
    return 0
end function

global function ᪠__ப__(sequence pattern, sequence string)
-- returns TRUE if string matches pattern
-- pattern can include '*' and '?' "wildcard" characters
    integer p, f, t 
    sequence match_string
    
    pattern = pattern & END_MARKER
    string = string & END_MARKER
    p = 1
    f = 1
    while f <= length(string) do
	if not find(pattern[p], {string[f], '?'}) then
	    if pattern[p] = '*' then
		while pattern[p] = '*' do
		    p += 1
		end while
		if pattern[p] = END_MARKER then
		    return 1
		end if
		match_string = ""
		while pattern[p] != '*' do
		    match_string = match_string & pattern[p]
		    if pattern[p] = END_MARKER then
			exit
		    end if
		    p += 1
		end while
		if pattern[p] = '*' then
		    p -= 1
		end if
		t = qmatch(match_string, string[f..length(string)])
		if t = 0 then
		    return 0
		else
		    f += t + length(match_string) - 2
		end if
	    else
		return 0
	    end if
	end if
	p += 1
	f += 1
	if p > length(pattern) then
	    return f > length(string) 
	end if
    end while
    return 0
end function

global function wildcard_file(sequence pattern, sequence filename)
-- Return 1 (TRUE) if filename matches the wild card pattern.
-- Similar to DOS wild card matching but better. For example, 
-- "*ABC.*" in DOS will match *all* files, where this function will 
-- only match when the file name part has "ABC" at the end.
    
    pattern = upper(pattern)
    filename = upper(filename)
    if not find('.', pattern) then
	pattern = pattern & '.'
    end if
    if not find('.', filename) then
	filename = filename & '.'
    end if
    return wildcard_match(pattern, filename)
end function

global function ᪠__䠩__
(sequence pattern, sequence filename)
-- Return 1 (TRUE) if filename matches the wild card pattern.
-- Similar to DOS wild card matching but better. For example, 
-- "*ABC.*" in DOS will match *all* files, where this function will 
-- only match when the file name part has "ABC" at the end.
    
    pattern = upper(pattern)
    filename = upper(filename)
    if not find('.', pattern) then
	pattern = pattern & '.'
    end if
    if not find('.', filename) then
	filename = filename & '.'
    end if
    return wildcard_match(pattern, filename)
end function

global function case_ru(integer c, object x, sequence cp)
-- convert Russian atom or sequence 
-- to upper or to lower case
 integer n
 sequence A, a
    if equal(cp,"dos") 
  then n=1
 elsif equal(cp,"win")
  then n=2
 elsif equal(cp,"koi") 
  then n=3
 elsif equal(cp,"iso") 
  then n=4
 elsif equal(cp,"mac")
  then n=5
  else n=0 end if 
 if not n then return n end if
 if c then 
     A=codes[n][ 1..32]
     a=codes[n][33..64]
 else 
     a=codes[n][ 1..32]
     A=codes[n][33..64]
 end if    
 if atom(x) then n=find(x,a) if n then return A[n] end if
 else
     for i=1 to length(x) do
	 n=find(x[i],a)
	 if n then
	    x[i] = A[n]
	 end if
     end for  
 end if      
return x
end function -- case_ru()

global function ᪨_ॣ(integer c, object x, sequence cp)
-- ८ࠧ ᪨ atom  sequence 
--  孥   ॣ
 integer n
 sequence A, a
    if equal(cp,"dos") 
  then n=1
 elsif equal(cp,"win")
  then n=2
 elsif equal(cp,"koi") 
  then n=3
 elsif equal(cp,"iso") 
  then n=4
 elsif equal(cp,"mac") 
  then n=5
  else n=0 end if 
 if not n then return n end if
 if c then 
     A=codes[n][ 1..32]
     a=codes[n][33..64]
 else 
     a=codes[n][ 1..32]
     A=codes[n][33..64]
 end if    
 if atom(x) then n=find(x,a) if n then return A[n] end if
 else
     for i=1 to length(x) do
	 n=find(x[i],a)
	 if n then
	    x[i] = A[n]
	 end if
     end for  
 end if      
return x
end function -- ᪨_ॣ()

global function case_la(integer c, object x)
-- convert Latin atom or sequence to upper or to lower case
 integer d d='a' - 'A'
 if c then return x - (x >= 'a' and x <= 'z') * d
      else return x + (x >= 'A' and x <= 'Z') * d end if
end function -- case_la()

global function ⨭᪨_ॣ(integer c, object x)
-- ८ࠧ ⨭᪨ atom  sequence  孥   ॣ
 integer d d='a' - 'A'
 if c then return x - (x >= 'a' and x <= 'z') * d
      else return x + (x >= 'A' and x <= 'Z') * d end if
end function -- ⨭᪨_ॣ()

global function recode(object f, sequence m)
-- recode Russian text from one encoding to another one
integer c,r,i,o
       i=m[1]  
    if i='d' then i=1
 elsif i='w' then i=2
 elsif i='k' then i=3
 elsif i='i' then i=4
 elsif i='m' then i=5 
  else            i=0 end if
       o=m[2]
    if o='d' then o=1
 elsif o='w' then o=2
 elsif o='k' then o=3
 elsif o='i' then o=4
 elsif o='m' then o=5 
 elsif o='l' then o=6 
  else            o=0 end if
	   if not i 
	   or not o 
	 then puts(1,"\nbad recodeing mode") 
	      abort(1) end if
    if atom(f) 
 then c=find(f,codes[i]) if c 
		       then return codes[o][c] 
		       else return f end if
 else
    for j=1 to length(f) do 
	c=f[j]
	r=c
	  if c>=128 
	then  c=find(c,codes[i])
	  if c 
	then f[j]=codes[o][c]
	else f[j]=r 
      end if 
      end if
    end for
  return f
 end if  
end function -- recode() --

global function ४஢(object f, sequence m)
--४ ᪨ ⥪   ஢  
integer c,r,i,o
       i=m[1]  
    if i='d' then i=1
 elsif i='w' then i=2
 elsif i='k' then i=3
 elsif i='i' then i=4
 elsif i='m' then i=5 
  else            i=0 end if
       o=m[2]
    if o='d' then o=1
 elsif o='w' then o=2
 elsif o='k' then o=3
 elsif o='i' then o=4
 elsif o='m' then o=5 
 elsif o='l' then o=6 
  else            o=0 end if
	   if not i 
	   or not o 
	 then puts(1,"\n ० ४஢") 
	      abort(1) end if
    if atom(f) 
  then c=find(f,codes[i]) if c 
		       then return codes[o][c] 
		       else return f end if
  else
     for j=1 to length(f) do 
	c=f[j]
	r=c
	  if c>=128 
	then  c=find(c,codes[i])
	  if c 
	then f[j]=codes[o][c]
	else f[j]=r 
      end if 
      end if
    end for
end if  
  return f
end function -- ४஢() --

--     include builtinr.e
--     뢥(1, ४஢("஡   ᨭ 뫠\n", "dl"))
--     뢥(1, recode(
--                ४஢(
--                "஡   ᨭ 뫠\n",
--                "dk"),
--                "kl"))

